/**
 *    @file  libhttp.h
 *   @brief  HTTP parsing and building library
 *
 *  <+DETAILED+>
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  19/11/2024
 *      Revision:  none
 * Last modified:  2024-11-20 22:08
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __LIBHTTP_H__
#define __LIBHTTP_H__

typedef struct HTTPHeader_s  HTTPHeader_t;

HTTPHeader_t* HTTPHeader_new(const char* name, const char* value);
void          HTTPHeader_del(HTTPHeader_t* header);
HTTPHeader_t* HTTPHeader_basicauth(const char* login, const char* pass);
char*         HTTPHeader_getname(HTTPHeader_t* header);
char*         HTTPHeader_getvalue(HTTPHeader_t* header);


typedef struct HTTP_s HTTP_t;

HTTP_t* HTTP_new();
void    HTTP_del(HTTP_t* http);
char*   HTTP_setbody(HTTP_t* http, const char* body);
char*   HTTP_getbody(HTTP_t* http);

/* API to implement
 *
 * typedef enum   HTTPStatus_e  HTTPStatus_t;
 * typedef enum   HTTPMethod_e  HTTPMethod_t;
 * typedef struct HTTPUri_s     HTTPUri_t;
 * typedef enum   HTTPVersion_e HTTPVersion_t;
 *
 * void          HTTP_addheader(HTTP_t*, HTTPHeader_t*);
 * void          HTTP_delheader(HTTP_t*, HTTPHeader_t*);
 * HTTPHeader_t* HTTP_firstheader(HTTP_t*);
 * HTTPHeader_t* HTTP_nextheader(HTTPHeader_t*);
 * HTTPHeader_t* HTTP_findheader(HTTPHeader_t*);
 *
 * char*         HTTP_buildrequest(HTTPMethod_t, HTTPUri_t*, HTTPVersion_t);
 * void          HTTP_parserequest(const char* request, HTTP_Method_t, HTTPUri_t*, HTTPVersion_t);
 * char*         HTTP_buildstatus(HTTPVersion_t, HTTPStatus_t);
 * void          HTTP_parsestatus(const char* reply, HTTPVersion_t*, HTTPStatus_t*);
 * char*         HTTP_buildcontents(HTTP_t*);
 * HTTP_t*       HTTP_parsecontents(const char* reply);
 */

typedef enum HTTPVersion_e {
    HTTPVERSION_HTTP09,
    HTTPVERSION_HTTP10,
    HTTPVERSION_HTTP11,
    HTTPVERSION_HTTP11b,
    HTTPVERSION_HTTP2,
    HTTPVERSION_HTTP3,
    HTTPVERSION_INVALID
} HTTPVersion_t;

typedef enum HTTPMethod_e {
    HTTPMETHOD_GET,
    HTTPMETHOD_HEAD,
    HTTPMETHOD_POST,
    HTTPMETHOD_PUT,
    HTTPMETHOD_DELETE,
    HTTPMETHOD_CONNECT,
    HTTPMETHOD_OPTIONS,
    HTTPMETHOD_TRACE,
    HTTPMETHOD_PATCH,
    HTTPMETHOD_INVALID
} HTTPMethod_t;

/* POC style API */
void HTTP_addheader(HTTP_t* http, const char* name, const char* value);
char* HTTP_getheaders(const HTTP_t* http);
char* HTTP_getrequest(const HTTPMethod_t method, const char* uri, const HTTPVersion_t version);


#endif /* __LIBHTTP_H__ */
